@@ -338,6 +338,22 @@ static uint64_t _get_hash_inx(mysql_conn_t *mysql_conn,
 
 }
 
+static int _create_tres_replace_str(void *x, void *args)
+{
+	slurmdb_tres_rec_t *tres_rec = x;
+	char **vals = args;
+
+	/*
+	 * If we already have the tres we are looking for we will replace it
+	 * with the new value, otherwise we will just tack it onto the end.
+	 */
+	xstrfmtcat(*vals, ", tres_alloc=case when tres_alloc regexp '(^|,)%u=' then regexp_replace(tres_alloc, '(^|,)(%u)=[[:digit:]]+', '\\\\1\\\\2=%"PRIu64"') else concat_ws(',', tres_alloc, '%u=%"PRIu64"') end",
+		   tres_rec->id, tres_rec->id, tres_rec->count,
+		   tres_rec->id, tres_rec->count);
+
+	return 0;
+}
+
 /* extern functions */
 
 extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
@@ -408,11 +424,9 @@ extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 	else
 		check_time = submit_time;
 
-	slurm_mutex_lock(&rollup_lock);
-	if (check_time < global_last_rollup) {
-		MYSQL_ROW row;
-
-		/* check to see if we are hearing about this time for the
+	if (trigger_reroll(mysql_conn, check_time)) {
+		/*
+		 * Check to see if we are hearing about this time for the
 		 * first time.
 		 */
 		query = xstrdup_printf("select job_db_inx "
@@ -423,63 +437,29 @@ extern int as_mysql_job_start(mysql_conn_t *mysql_conn, job_record_t *job_ptr)
 				       job_table, job_ptr->job_id,
 				       submit_time, begin_time, start_time);
 		DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
-		if (!(result =
-		      mysql_db_query_ret(mysql_conn, query, 0))) {
+		if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 			xfree(query);
-			slurm_mutex_unlock(&rollup_lock);
 			return SLURM_ERROR;
 		}
 		xfree(query);
-		if ((row = mysql_fetch_row(result))) {
-			mysql_free_result(result);
-			debug4("revieved an update for a "
-			       "job (%u) already known about",
+		if (mysql_fetch_row(result))
+			debug4("Received an update for a job (%u) already known about",
 			       job_ptr->job_id);
-			slurm_mutex_unlock(&rollup_lock);
-			goto no_rollup_change;
-		}
-		mysql_free_result(result);
-
-		if (job_ptr->start_time)
-			debug("Need to reroll usage from %s Job %u "
-			      "from %s started then and we are just "
-			      "now hearing about it.",
+		else if (job_ptr->start_time)
+			debug("Need to reroll usage from %s Job %u from %s started then and we are just now hearing about it.",
 			      slurm_ctime2(&check_time),
 			      job_ptr->job_id, mysql_conn->cluster_name);
 		else if (begin_time)
-			debug("Need to reroll usage from %s Job %u "
-			      "from %s became eligible then and we are just "
-			      "now hearing about it.",
+			debug("Need to reroll usage from %s Job %u from %s became eligible then and we are just now hearing about it.",
 			      slurm_ctime2(&check_time),
 			      job_ptr->job_id, mysql_conn->cluster_name);
 		else
-			debug("Need to reroll usage from %s Job %u "
-			      "from %s was submitted then and we are just "
-			      "now hearing about it.",
+			debug("Need to reroll usage from %s Job %u from %s was submitted then and we are just now hearing about it.",
 			      slurm_ctime2(&check_time),
 			      job_ptr->job_id, mysql_conn->cluster_name);
 
-		global_last_rollup = check_time;
-		slurm_mutex_unlock(&rollup_lock);
-
-		/* If the times here are later than the daily_rollup
-		   or monthly rollup it isn't a big deal since they
-		   are always shrunk down to the beginning of each
-		   time period.
-		*/
-		query = xstrdup_printf("update \"%s_%s\" set "
-				       "hourly_rollup=%ld, "
-				       "daily_rollup=%ld, monthly_rollup=%ld",
-				       mysql_conn->cluster_name,
-				       last_ran_table, check_time,
-				       check_time, check_time);
-		DB_DEBUG(DB_JOB, mysql_conn->conn, "query\n%s", query);
-		rc = mysql_db_query(mysql_conn, query);
-		xfree(query);
-	} else
-		slurm_mutex_unlock(&rollup_lock);
-
-no_rollup_change:
+		mysql_free_result(result);
+	}
 
 	if (job_ptr->name && job_ptr->name[0])
 		jname = job_ptr->name;
@@ -512,7 +492,7 @@ no_rollup_change:
 		if (!(job_ptr->db_index = _get_db_index(mysql_conn,
 							submit_time,
 							job_ptr->job_id)))
-			job_record_set_sluid(job_ptr);
+			job_record_set_sluid(job_ptr, false);
 	}
 
 	if (!IS_JOB_IN_DB(job_ptr)) {
@@ -988,6 +968,14 @@ extern list_t *as_mysql_modify_job(mysql_conn_t *mysql_conn, uint32_t uid,
 	if (job->extra)
 		xstrfmtcat(vals, ", extra='%s'", job->extra);
 
+	if (job->tres_alloc_str) {
+		list_t *tmp_list = NULL;
+		slurmdb_tres_list_from_string(&tmp_list, job->tres_alloc_str,
+					      TRES_STR_FLAG_NONE, NULL);
+		(void) list_for_each(tmp_list, _create_tres_replace_str, &vals);
+		FREE_NULL_LIST(tmp_list);
+	}
+
 	if (job->wckey)
 		xstrfmtcat(vals, ", wckey='%s'", job->wckey);
 
@@ -1023,6 +1011,15 @@ extern list_t *as_mysql_modify_job(mysql_conn_t *mysql_conn, uint32_t uid,
 			break;
 		}
 
+		/* If we changed the TRES we need to reroll the usage */
+		if (job->tres_alloc_str && job_rec->start) {
+			if (trigger_reroll(mysql_conn, job_rec->start)) {
+				debug("Need to reroll usage from %s Job %u from %s started then and we just changed the TRES on it.",
+				      slurm_ctime2(&job_rec->start),
+				      job_rec->jobid,
+				      mysql_conn->cluster_name);
+			}
+		}
 		slurm_make_time_str(&job_rec->submit,
 				    tmp_char, sizeof(tmp_char));
 
