@@ -378,7 +378,10 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 	if (!list_args.list) {
 		const parser_t *const lparser =
 			find_parser_by_type(parser->list_type);
-		list_args.list = list_create((ListDelF) lparser->free);
+		if (lparser->free)
+			list_args.list = list_create((ListDelF) lparser->free);
+		else
+			list_args.list = list_create(xfree_ptr);
 	}
 
 	xassert(list_count(list_args.list) >= 0);
