@@ -242,7 +242,7 @@ static void _set_procdatas(list_t *lresp)
 
 		/* localid is the task id in this node */
 		localid = pmixp_info_taskid2localid(i);
-		if (0 <= localid) {
+		if (localid >= 0) {
 			PMIXP_KVP_CREATE(kvp, PMIX_LOCAL_RANK,
 					 &localid, PMIX_UINT16);
 			list_append(rankinfo, kvp);
@@ -332,60 +332,6 @@ static void _set_sizeinfo(list_t *lresp)
 	list_append(lresp, kvp);
 }
 
-/*
- * provide topology information if hwloc is available
- */
-static void _set_topology(list_t *lresp)
-{
-#ifdef HAVE_HWLOC
-	hwloc_topology_t topology;
-	unsigned long flags;
-	pmix_info_t *kvp;
-	char *p = NULL;
-	int len;
-
-	if (0 != hwloc_topology_init(&topology)) {
-		/* error in initialize hwloc library */
-		error("%s: hwloc_topology_init() failed", __func__);
-		goto err_exit;
-	}
-
-#if HWLOC_API_VERSION < 0x00020000
-	flags = (HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM |
-		 HWLOC_TOPOLOGY_FLAG_IO_DEVICES);
-	hwloc_topology_set_flags(topology, flags);
-#else
-	flags = HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM;
-	hwloc_topology_set_flags(topology, flags);
-	hwloc_topology_set_io_types_filter(topology,
-					   HWLOC_TYPE_FILTER_KEEP_ALL);
-#endif
-
-	if (hwloc_topology_load(topology)) {
-		error("%s: hwloc_topology_load() failed", __func__);
-		goto err_release_topo;
-	}
-
-#if HWLOC_API_VERSION < 0x00020000
-	if (0 != hwloc_topology_export_xmlbuffer(topology, &p, &len)) {
-#else
-	if (0 != hwloc_topology_export_xmlbuffer(topology, &p, &len, 0)) {
-#endif
-		error("%s: hwloc_topology_load() failed", __func__);
-		goto err_release_topo;
-	}
-
-	PMIXP_KVP_CREATE(kvp, PMIX_LOCAL_TOPO, p, PMIX_STRING);
-	list_append(lresp, kvp);
-
-	/* successful exit - fallthru */
-err_release_topo:
-	hwloc_topology_destroy(topology);
-err_exit:
-#endif
-	return;
-}
-
 /*
  * Estimate the size of a buffer capable of holding the proc map for this job.
  * PMIx proc map string format:
@@ -454,7 +400,7 @@ static int _set_mapsinfo(list_t *lresp)
 	input = hostlist_deranged_string_xmalloc(hl);
 	rc = PMIx_generate_regex(input, &regexp);
 	xfree(input);
-	if (PMIX_SUCCESS != rc) {
+	if (rc != PMIX_SUCCESS) {
 		return SLURM_ERROR;
 	}
 	PMIXP_KVP_CREATE(kvp, PMIX_NODE_MAP, regexp, PMIX_STRING);
@@ -484,7 +430,7 @@ static int _set_mapsinfo(list_t *lresp)
 	xfree(map);
 	xfree(node2tasks);
 
-	if (PMIX_SUCCESS != rc) {
+	if (rc != PMIX_SUCCESS) {
 		return SLURM_ERROR;
 	}
 
@@ -526,21 +472,24 @@ static void _set_localinfo(list_t *lresp)
 extern int pmixp_libpmix_init(void)
 {
 	int rc;
+	bool trusted;
 
-	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_lib()))) {
+	trusted = (pmixp_info_flags() & PMIXP_FLAG_TRUSTED_LIB_TMPDIR);
+	if ((rc = pmixp_mkdir(pmixp_info_tmpdir_lib(), trusted))) {
 		PMIXP_ERROR_STD("Cannot create server lib tmpdir: \"%s\"",
 				pmixp_info_tmpdir_lib());
 		return errno;
 	}
 
-	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_cli()))) {
+	trusted = (pmixp_info_flags() & PMIXP_FLAG_TRUSTED_CLI_TMPDIR);
+	if ((rc = pmixp_mkdir(pmixp_info_tmpdir_cli(), trusted))) {
 		PMIXP_ERROR_STD("Cannot create client cli tmpdir: \"%s\"",
 				pmixp_info_tmpdir_cli());
 		return errno;
 	}
 
 	rc = pmixp_lib_init();
-	if (SLURM_SUCCESS != rc) {
+	if (rc != SLURM_SUCCESS) {
 		PMIXP_ERROR_STD("PMIx_server_init failed with error %d\n", rc);
 		return SLURM_ERROR;
 	}
@@ -558,14 +507,14 @@ extern int pmixp_libpmix_finalize(void)
 	rc = pmixp_lib_finalize();
 
 	rc1 = rmdir_recursive(pmixp_info_tmpdir_lib(), true);
-	if (0 != rc1) {
+	if (rc1) {
 		PMIXP_ERROR_STD("Failed to remove %s\n",
 				pmixp_info_tmpdir_lib());
 		/* Not considering this as fatal error */
 	}
 
 	rc1 = rmdir_recursive(pmixp_info_tmpdir_cli(), true);
-	if (0 != rc1) {
+	if (rc1) {
 		PMIXP_ERROR_STD("Failed to remove %s\n",
 				pmixp_info_tmpdir_cli());
 		/* Not considering this as fatal error */
@@ -619,7 +568,7 @@ extern int pmixp_lib_dmodex_request(
 	strlcpy(proc_v1.nspace, proc->nspace, PMIX_MAX_NSLEN);
 
 	rc = PMIx_server_dmodex_request(&proc_v1, cbfunc, caddy);
-	if (PMIX_SUCCESS != rc) {
+	if (rc != PMIX_SUCCESS) {
 		return SLURM_ERROR;
 	}
 	return SLURM_SUCCESS;
@@ -633,7 +582,7 @@ extern int pmixp_lib_setup_fork(uint32_t rank, const char *nspace, char ***env)
 	proc.rank = rank;
 	strlcpy(proc.nspace, nspace, PMIX_MAX_NSLEN);
 	rc = PMIx_server_setup_fork(&proc, env);
-	if (PMIX_SUCCESS != rc) {
+	if (rc != PMIX_SUCCESS) {
 		return SLURM_ERROR;
 	}
 	return SLURM_SUCCESS;
@@ -682,11 +631,9 @@ extern int pmixp_libpmix_job_set(void)
 
 	_set_sizeinfo(lresp);
 
-	_set_topology(lresp);
-
 	_set_euid(lresp);
 
-	if (SLURM_SUCCESS != _set_mapsinfo(lresp)) {
+	if (_set_mapsinfo(lresp) != SLURM_SUCCESS) {
 		FREE_NULL_LIST(lresp);
 		PMIXP_ERROR("Can't build nodemap");
 		return SLURM_ERROR;
@@ -710,7 +657,7 @@ extern int pmixp_libpmix_job_set(void)
 					 ninfo, _release_cb,
 					 &register_caddy[0]);
 
-	if (PMIX_SUCCESS != rc) {
+	if (rc != PMIX_SUCCESS) {
 		PMIXP_ERROR("Cannot register namespace %s, nlocalproc=%d, ninfo = %d",
 			    pmixp_info_namespace(), pmixp_info_tasks_loc(),
 			    ninfo);
@@ -726,7 +673,7 @@ extern int pmixp_libpmix_job_set(void)
 		rc = PMIx_server_register_client(&proc, uid, gid, NULL,
 						 _release_cb,
 						 &register_caddy[i + 1]);
-		if (PMIX_SUCCESS != rc) {
+		if (rc != PMIX_SUCCESS) {
 			PMIXP_ERROR("Cannot register client %d(%d) in namespace %s",
 				    pmixp_info_taskid(i), i,
 				    pmixp_info_namespace());
@@ -756,7 +703,7 @@ extern int pmixp_libpmix_job_set(void)
 					exit_flag = 0;
 				}
 				// An error may occur during registration
-				if (PMIX_SUCCESS != register_caddy[i].rc) {
+				if (register_caddy[i].rc != PMIX_SUCCESS) {
 					PMIXP_ERROR("Failed to complete registration #%d, error: %d", i, register_caddy[i].rc);
 					ret = SLURM_ERROR;
 				}
@@ -807,7 +754,7 @@ extern int pmixp_lib_fence(const pmix_proc_t procs[], size_t nprocs,
 		goto error;
 	}
 	ret = pmixp_coll_contrib_local(coll, type, data, ndata, cbfunc, cbdata);
-	if (SLURM_SUCCESS != ret) {
+	if (ret != SLURM_SUCCESS) {
 		status = PMIX_ERROR;
 		goto error;
 	}
@@ -832,8 +779,7 @@ extern int pmixp_lib_abort(int status, void *cbfunc, void *cbdata)
 	if (!status)
 		flags |= KILL_NO_SIG_FAIL;
 
-	slurm_kill_job_step(pmixp_info_jobid(), pmixp_info_stepid(), SIGKILL,
-			    flags);
+	slurm_kill_job_step(pmixp_info_step_id(), SIGKILL, flags);
 
 	if (abort_cbfunc)
 		abort_cbfunc(PMIX_SUCCESS, cbdata);
