@@ -79,6 +79,7 @@ const uint32_t plugin_version = SLURM_VERSION_NUMBER;
 
 static ns_conf_t *ns_conf = NULL;
 static bool plugin_disabled = false;
+static pid_t ns_pid = -1;
 
 /* NS_L_NS must be last */
 enum ns_l_types {
@@ -96,7 +97,9 @@ typedef struct {
 	char *proc_name;
 } ns_l_t;
 
-static ns_l_t ns_l_enabled[NS_L_END] = { { false, -1, 0, NULL, NULL } };
+static ns_l_t ns_l_enabled[NS_L_END] = { { false, -1, 0, NULL, NULL },
+					 { false, -1, 0, NULL, NULL },
+					 { false, -1, 0, NULL, NULL } };
 
 static void _create_paths(uint32_t job_id, char **job_mount, char **ns_base,
 			  char **src_bind)
@@ -117,7 +120,7 @@ static void _create_paths(uint32_t job_id, char **job_mount, char **ns_base,
 		}
 		if (ns_conf->clonensflags & CLONE_NEWPID) {
 			ns_l_enabled[NS_L_PID].enabled = true;
-			ns_l_enabled[NS_L_NS].flag = CLONE_NEWPID;
+			ns_l_enabled[NS_L_PID].flag = CLONE_NEWPID;
 			xfree(ns_l_enabled[NS_L_PID].path);
 			xstrfmtcat(ns_l_enabled[NS_L_PID].path, "%s/pid",
 				   *ns_base);
@@ -125,7 +128,7 @@ static void _create_paths(uint32_t job_id, char **job_mount, char **ns_base,
 		}
 		if (ns_conf->clonensflags & CLONE_NEWUSER) {
 			ns_l_enabled[NS_L_USER].enabled = true;
-			ns_l_enabled[NS_L_NS].flag = CLONE_NEWUSER;
+			ns_l_enabled[NS_L_USER].flag = CLONE_NEWUSER;
 			xfree(ns_l_enabled[NS_L_USER].path);
 			xstrfmtcat(ns_l_enabled[NS_L_USER].path, "%s/user",
 				   *ns_base);
@@ -178,7 +181,7 @@ static int _restore_ns(list_t *steps, const char *d_name)
 		return _delete_ns(job_id);
 	}
 
-	close(fd);
+	fd_close(&fd);
 
 	return SLURM_SUCCESS;
 }
@@ -209,8 +212,7 @@ extern void fini(void)
 #ifdef MEMORY_LEAK_DEBUG
 	for (int i = 0; i < NS_L_END; i++) {
 		xfree(ns_l_enabled[i].path);
-		if (ns_l_enabled[i].fd >= 0)
-			close(ns_l_enabled[i].fd);
+		fd_close(&ns_l_enabled[i].fd);
 	}
 	free_ns_conf();
 #endif
@@ -546,7 +548,7 @@ child_exit:
 
 static int _clonens_user_setup(stepd_step_rec_t *step, pid_t pid)
 {
-	int fd = 0, rc = SLURM_SUCCESS;
+	int fd = -1, rc = SLURM_SUCCESS;
 	char *tmpstr = NULL;
 
 	if (!ns_l_enabled[NS_L_USER].enabled)
@@ -591,8 +593,7 @@ static int _clonens_user_setup(stepd_step_rec_t *step, pid_t pid)
 		rc = SLURM_ERROR;
 		goto end_it;
 	}
-	if (fd >= 0)
-		close(fd);
+	fd_close(&fd);
 	xfree(tmpstr);
 
 	xstrfmtcat(tmpstr, "/proc/%d/gid_map", pid);
@@ -609,8 +610,7 @@ static int _clonens_user_setup(stepd_step_rec_t *step, pid_t pid)
 	}
 
 end_it:
-	if (fd >= 0)
-		close(fd);
+	fd_close(&fd);
 	xfree(tmpstr);
 	return rc;
 }
@@ -625,7 +625,6 @@ static int _create_ns(stepd_step_rec_t *step)
 	unsigned long tls = 0;
 	sem_t *sem1 = NULL;
 	sem_t *sem2 = NULL;
-	pid_t cpid;
 
 	_create_paths(step->step_id.job_id, &job_mount, &ns_base, &src_bind);
 
@@ -670,7 +669,7 @@ static int _create_ns(stepd_step_rec_t *step)
 			rc = -1;
 			goto exit2;
 		}
-		close(fd);
+		fd_close(&fd);
 	}
 
 	/* Create location for bind mounts to go */
@@ -739,14 +738,14 @@ static int _create_ns(stepd_step_rec_t *step)
 		error("%s: sem_init: %m", __func__);
 		goto exit1;
 	}
-	cpid = sys_clone(ns_conf->clonensflags|SIGCHLD, &parent_tid,
-			 &child_tid, tls);
+	ns_pid = sys_clone(ns_conf->clonensflags | SIGCHLD, &parent_tid,
+			   &child_tid, tls);
 
-	if (cpid == -1) {
+	if (ns_pid == -1) {
 		error("%s: sys_clone failed: %m", __func__);
 		rc = -1;
 		goto exit1;
-	} else if (cpid == 0) {
+	} else if (ns_pid == 0) {
 		_create_ns_child(step, src_bind, job_mount, sem1, sem2);
 	} else {
 		char *proc_path = NULL;
@@ -758,7 +757,7 @@ static int _create_ns(stepd_step_rec_t *step)
 		for (int i = 0; i < NS_L_END; i++) {
 			if (!ns_l_enabled[i].enabled)
 				continue;
-			xstrfmtcat(proc_path, "/proc/%u/ns/%s", cpid,
+			xstrfmtcat(proc_path, "/proc/%u/ns/%s", ns_pid,
 				   ns_l_enabled[i].proc_name);
 			rc = mount(proc_path, ns_l_enabled[i].path, NULL,
 				   MS_BIND, NULL);
@@ -775,7 +774,7 @@ static int _create_ns(stepd_step_rec_t *step)
 		}
 
 		/* setup users before setting up the rest of the container */
-		if ((rc = _clonens_user_setup(step, cpid))) {
+		if ((rc = _clonens_user_setup(step, ns_pid))) {
 			error("%s: Unable to prepare user namespace.",
 			      __func__);
 			/* error needs to fall though here */
@@ -794,9 +793,9 @@ static int _create_ns(stepd_step_rec_t *step)
 			goto exit1;
 		}
 
-		if (proctrack_g_add(step, cpid) != SLURM_SUCCESS) {
+		if (proctrack_g_add(step, ns_pid) != SLURM_SUCCESS) {
 			error("%s: Job %u can't add pid %d to proctrack plugin in the extern_step.",
-			      __func__, step->step_id.job_id, cpid);
+			      __func__, step->step_id.job_id, ns_pid);
 			rc = SLURM_ERROR;
 			goto exit1;
 		}
@@ -884,7 +883,7 @@ extern int namespace_p_join_external(slurm_step_id_t *step_id, list_t *ns_map)
 		if (!ns_l_enabled[i].enabled)
 			continue;
 
-		if (!ns_l_enabled[i].fd) {
+		if (ns_l_enabled[i].fd == -1) {
 			ns_l_enabled[i].fd =
 				open(ns_l_enabled[i].path, O_RDONLY);
 			if (ns_l_enabled[i].fd == -1) {
@@ -947,8 +946,7 @@ extern int namespace_p_join(slurm_step_id_t *step_id, uid_t uid,
 		if (!ns_l_enabled[i].enabled)
 			continue;
 		rc = setns(ns_l_enabled[i].fd, 0);
-		close(ns_l_enabled[i].fd);
-		ns_l_enabled[i].fd = -1;
+		fd_close(&ns_l_enabled[i].fd);
 		if (rc) {
 			error("%s: setns failed for %s: %m",
 			      __func__, ns_l_enabled[i].path);
@@ -1058,6 +1056,17 @@ extern int namespace_p_stepd_delete(slurm_step_id_t *step_id)
 	if (plugin_disabled)
 		return SLURM_SUCCESS;
 
+	if (ns_pid) {
+		int wstatus;
+		/*
+		 * The namespace process may have been signaled already, but
+		 * kill it to be sure.
+		 */
+		kill(ns_pid, SIGKILL);
+		waitpid(ns_pid, &wstatus, 0);
+		ns_pid = -1;
+	}
+
 	return _delete_ns(step_id->job_id);
 }
 
@@ -1162,7 +1171,6 @@ extern int namespace_p_setup_bpf_token(stepd_step_rec_t *step)
 		rc = SLURM_SUCCESS;
 	}
 end:
-	if (fd >= 0)
-		close(fd);
+	fd_close(&fd);
 	return rc;
 }
